<?php
/*
 +--------------------------------------------------------------------------+
 | This file is part of the Kolab Web Admin Panel                           |
 |                                                                          |
 | Copyright (C) 2011-2012, Kolab Systems AG                                |
 |                                                                          |
 | This program is free software: you can redistribute it and/or modify     |
 | it under the terms of the GNU Affero General Public License as published |
 | by the Free Software Foundation, either version 3 of the License, or     |
 | (at your option) any later version.                                      |
 |                                                                          |
 | This program is distributed in the hope that it will be useful,          |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             |
 | GNU Affero General Public License for more details.                      |
 |                                                                          |
 | You should have received a copy of the GNU Affero General Public License |
 | along with this program. If not, see <http://www.gnu.org/licenses/>      |
 +--------------------------------------------------------------------------+
 | Author: Aleksander Machniak <machniak@kolabsys.com>                      |
 +--------------------------------------------------------------------------+
*/

class kolab_client_task_resource extends kolab_client_task
{
    protected $ajax_only = true;

    protected $menu = array(
        'add'  => 'resource.add',
    );

    protected $list_attribs = array('cn');

    /**
     * Default action.
     */
    public function action_default()
    {
        $this->output->set_object('content', 'resource', true);
        $this->output->set_object('task_navigation', $this->menu());

        $this->action_list();

        // display form to add resource if logged-in user has right to do so
        $caps = $this->get_capability('actions');
        if (!empty($caps['resource.add'])) {
            $this->action_add();
        }
        else {
            $this->output->command('set_watermark', 'taskcontent');
        }
    }

    /**
     * Resource adding (form) action.
     */
    public function action_add()
    {
        $data   = $this->get_input('data', 'POST');
        $output = $this->resource_form(null, $data, true);

        $this->output->set_object('taskcontent', $output);
    }

    /**
     * Resource information (form) action.
     */
    public function action_info()
    {
        $id       = $this->get_input('id', 'POST');
        $result   = $this->api_get('resource.info', array('id' => $id));
        $resource = $result->get();
        $output   = $this->resource_form(null, $resource);

        $this->output->set_object('taskcontent', $output);
    }

    private function resource_form($attribs, $data = array())
    {
        if (empty($attribs['id'])) {
            $attribs['id'] = 'resource-form';
        }

        // Form sections
        $sections = array(
            'system'        => 'resource.system',
            'other'         => 'resource.other',
        );

        // field-to-section map and fields order
        $fields_map = array(
            'type_id'                   => 'system',
            'type_id_name'              => 'system',

            'cn'                        => 'system',
            'ou'                        => 'system',
            'preferredlanguage'         => 'system',

            'mail'                      => 'system',
            'alias'                     => 'system',
            'mailalternateaddress'      => 'system',

            'owner'                     => 'system',

            'member'                    => 'system',
            'uniquemember'              => 'system',
            'memberurl'                 => 'system',

            'nsrole'                    => 'system',
            'nsroledn'                  => 'system',

            /* Kolab Settings */
            'kolabhomeserver'           => 'system',
            'mailhost'                  => 'system',
            'mailquota'                 => 'system',
            'kolabfreebusyfuture'       => 'system',
            'kolabinvitationpolicy'     => 'system',
            'kolabdelegate'             => 'system',
            'kolaballowsmtprecipient'   => 'system',
            'kolaballowsmtpsender'      => 'system',
            'kolabtargetfolder'         => 'system',

            'kolabdescattribute'        => 'other',
        );

        // Prepare fields
        $form = $this->form_prepare('resource', $data, array(), null, $fields_map['type_id']);
        list($fields, $types, $type, $add_mode) = $form;

        // Create form object and populate with fields
        $form = $this->form_create('resource', $attribs, $sections, $fields, $fields_map, $data, $add_mode);

        return $form->output();
    }
}
